@@ -1,7 +1,7 @@
 package Apache::CIPP;
 
-$VERSION = "0.11";
-$REVISION = q$Revision: 1.2 $;
+$VERSION = "0.13";
+$REVISION = q$Revision: 1.4 $;
 
 use strict;
 
@@ -18,6 +18,19 @@ use File::Path;
 
 sub handler {
 	my $r = shift;
+	
+	# should we mangle remote_ip with proxy X-Forwarded-For?
+	# (thanks to Ask Bjoern Hansen for his mod_proxy_add_forward.c.
+	#  this code snippet is stolen from his documentation ;)
+
+	if ( $r->dir_config ("mangle_proxy_remote_ip") ) {
+		if ( $r->connection->remote_ip eq '127.0.0.1' ) {
+			if ( my ($ip) = $r->header_in('X-Forwarded-For') =~
+				/([^,\s]+)$/ ) {
+				$r->connection->remote_ip($ip);
+			}
+		}
+	}
 
 	# print listing if a directory is requested
 
@@ -265,19 +278,64 @@ sub execute {
 	my $sub_name = $self->{sub_name};
 	my $r = $self->{r};
 	
+	$CIPP::REVISION =~ /(\d+\.\d+)/;
+	my $cipp_revision = $1;
+	$Apache::CIPP::REVISION =~ /(\d+\.\d+)/;
+	my $CIPP_revision = $1;
+
+	# special handling if we want to determine the content length
+	
+	if ( $r->dir_config ("determine_content_length") and
+	     $Apache::CIPP::mime_type{$sub_name} ne 'cipp/dynamic' ) {
+
+		if ( not $self->execute_sub_to_file ) {
+			# runtime error
+			$r->content_type ("text/html");
+			$r->send_http_header;
+			open (TMP, $self->{sub_temp_file});
+			$r->send_fd(\*TMP);
+			close (TMP);
+
+			return;
+		}
+		
+		$r->content_type ("text/html");
+		$r->header_out ("Content-Length", -s $self->{sub_temp_file} );
+		$r->send_http_header;
+		$r->print ("<!-- generated by CIPP $CIPP::VERSION/$cipp_revision with ".
+		   "Apache::CIPP $Apache::CIPP::VERSION/$CIPP_revision ".
+		   "-->\n");
+		   
+		open (TMP, $self->{sub_temp_file});
+		$r->send_fd(\*TMP);
+		close (TMP);
+		
+		return 1;
+	}
+	
+	# send simple http headers if the CIPP program wants us to
+	# send the header
+	
 	if ( $Apache::CIPP::mime_type{$sub_name} ne 'cipp/dynamic' ) {
-		$CIPP::REVISION =~ /(\d+\.\d+)/;
-		my $cipp_revision = $1;
-		$Apache::CIPP::REVISION =~ /(\d+\.\d+)/;
-		my $CIPP_revision = $1;
 
 		$r->content_type ("text/html");
+#		$r->header_out ("Expires", "Wed, 29 Mar 2000 14:24:35 GMT");
+#		$r->header_out ("Content-Length", 99 );
 		$r->send_http_header;
 		$r->print ("<!-- generated by CIPP $CIPP::VERSION/$cipp_revision with ".
 		   "Apache::CIPP $Apache::CIPP::VERSION/$CIPP_revision ".
 		   "-->\n");
 	}
-		
+	
+	return $self->execute_sub;
+}
+
+sub execute_sub {
+	my $self = shift;
+
+	my $sub_name = $self->{sub_name};
+	my $r = $self->{r};
+	
 	no strict 'refs';
 	eval { &$sub_name ($r) };
 	
@@ -285,10 +343,54 @@ sub execute {
 		$self->{error} = "runtime\t$@";
 		return;
 	}
-
+	
 	return 1;
 }
 
+sub execute_sub_to_file {
+	my $self = shift;
+
+	my $sub_name = $self->{sub_name};
+	my $r = $self->{r};
+	
+	# redirect STDOUT
+	
+	my $save_stdout = "STD$$";
+  
+	my $temp_file = "/tmp/apache-cipp.tmp$$";
+	$self->{sub_temp_file} = $temp_file;
+	  
+	{
+		no strict 'refs';
+		open ($save_stdout, ">& STDOUT")
+			or die "dup\tcan't dup STDOUT";
+	}
+
+	close (STDOUT) or die "redirect\tcan't close STDOUT";
+	open (STDOUT, "> $temp_file")
+  		or die "redirect\tcan't redirect STDOUT to '$temp_file'";
+	
+	# execute code
+	
+	no strict 'refs';
+	eval { &$sub_name ($r) };
+	my $error = $@;
+
+	# restore STDOUT
+	{
+		no strict 'refs';
+		open (STDOUT, ">& $save_stdout")
+			or die "dup\tcan't restore original STDOUT";
+		close $save_stdout;
+	}
+	
+	if ( $error ) {
+		$self->{error} = "runtime\t$@";
+		return;
+	}
+	
+	return 1;
+}
 
 sub error {
 	my $self = shift;
@@ -359,6 +461,8 @@ sub set_sub_filename {
 	my $filename = $self->{uri};
 	my $cache_dir = $self->{cache_dir};
 	
+	$cache_dir .= "/$ENV{SERVER_NAME}_$ENV{SERVER_PORT}";
+	
 	my $dir = $filename;
 	$dir =~ s![^/]+$!!;
 	$dir = $cache_dir.$dir;
@@ -375,6 +479,9 @@ sub set_sub_name {
 	my $self = shift;
 	
 	my $uri = $self->{uri};
+	
+	$uri .= "/$ENV{SERVER_NAME}_$ENV{SERVER_PORT}";
+	
 	$uri =~ s!^/!!;
 	$uri =~ s/\W/_/g;
 	
